'''
Created on 2014.03.27.

Sample program for Edaq530 measurements with plot.
The plot is created using gnuplot.
The program stops after the time specified by length elapses,
then saves the data to the file and shows the plot.
For more help run the program with '-h' argument.

'''


import edaqcomm
import sys
import pylab
import Gnuplot
import argparse
import os
import stat

DEFAULT_LENGTH = 10.0
DEFAULT_SAMPLE_RATE = 32

INVALID_CHANNEl_TPYE_MSG = "Invalid channeltype argument %s.\nValid arguments are: voltage, resistance, temp_kelvin, temp_celsius, inamp, photo.\nThe program now exit..."

class GnuplotEdaqMeas:

	'''
	Sets attributes related to channel type, uses a string
	'''
	def argsToChType(self, arg):
		if arg == "temp_celsius":
			self.chType = edaqcomm.ChannelType.resistor
			self.qType = edaqcomm.QuantityType.temp_celsius
			self.yLabel = "Temperature [C]"
		elif arg == "temp_kelvin":
			self.chType = edaqcomm.ChannelType.resistor
			self.qType = edaqcomm.QuantityType.temp_kelvin
			self.yLabel = "Temperature [K]"
		elif arg == "resistance":
			self.chType = edaqcomm.ChannelType.resistor
			self.qType = edaqcomm.QuantityType.resitance
			self.yLabel = "Resistance [Ohm]"
		elif arg == "voltage":
			self.chType = edaqcomm.ChannelType.voltage
			self.qType = edaqcomm.QuantityType.voltage
			self.yLabel = "Voltage [V]"
		elif arg == "inamp":
			self.chType = edaqcomm.ChannelType.inamp
			self.qType = edaqcomm.QuantityType.voltage
			self.yLabel = "Voltage [V]"
		elif arg == "photo":
			self.chType = edaqcomm.ChannelType.photo
			self.qType = edaqcomm.QuantityType.voltage
			self.yLabel = "Voltage [V]"
		else:
			print(INVALID_CHANNEl_TPYE_MSG % (arg))
			sys.exit(1)	
    	
    	'''
    	Sets the connection to the device
    	'''
    	def setConnection(self):
    		
    		self.argsToChType(self.args.chType)
    		self.comm = edaqcomm.Edaq530Comm(self.args.com)
		self.comm.connect()
		print(self.comm.getID())
		
		self.comm.setInputs(self.chType,
			edaqcomm.ChannelType.voltage, edaqcomm.ChannelType.voltage)
			
		self.comm.setSamplingFreq(self.args.sampleRate)
	
	'''
	Reads the ammount of data specified by length
	'''		
  	def readData(self):
  		self.Y = []
		temp = self.comm.fillArrayContSamplingOneCh(0, self.count)
		for i in range(0, self.count):
			self.Y.append(edaqcomm.getMeasuredQuantity(temp[i], self.qType))
  		self.X = pylab.np.linspace(0.0, self.args.length, self.count, endpoint=False)
  	
  	'''
  	Writes the measured data to the file
  	'''		
  	def writeToFile(self):
  		f = open(self.args.file, 'w+')
  		f.write('Time [s]\t%s\n' % self.yLabel)
  		for i in range(0, self.count):
  			f.write('%f\t%f\n' % (self.X[i], self.Y[i]))
  		f.write(' ')
  		f.close()
  		
  		# Set permisiions, needed because of sudo
  		os.chmod(self.args.file, stat.S_IWUSR | stat.S_IWGRP |
  		 stat.S_IWOTH | stat.S_IRUSR | stat.S_IRGRP |
  		 stat.S_IROTH)			

	'''
	Draws the gnuplot plot
	'''
	def drawPlot(self):
		g = Gnuplot.Gnuplot(debug=0, persist = 1)
		g('set style data lines')
  		g.ylabel(self.yLabel)
  		g.xlabel('Time [s]')
		d = Gnuplot.Data(self.X, self.Y)
		g.plot(d)

	'''
	Stops the measurement, disconnects the device
	'''
	def stop(self):
		self.comm.stopContSampling()
		self.comm.disconnect()
	
	'''
	Parse arguments using argparse
	'''	
	def parseArgs(self):
		self.parser = argparse.ArgumentParser(description = 'One channel Edaq530\
		 measurement. Measures for the sepecified amount of time, then writes\
		  the data to file and shows a gnuplot plot.')
		 
		self.parser.add_argument('com', help='the name of the com port,\
		 for example /dev/ttyUSB0')
		 
		self.parser.add_argument('file', help='file to store the measured data')
		 
		self.parser.add_argument('-cht', '--chtype', help='channel type, voltage\
		 by default. Possible arguments are: voltage, voltage_diff,\
		 resistance, temp_kelvin, temp_celsius, inamp, photo', 
			default='voltage', dest='chType')
		self.parser.add_argument('-sr', '--samplerate', help='device sample rate,\
		  %d Hz by default' % DEFAULT_SAMPLE_RATE,
			default=DEFAULT_SAMPLE_RATE, type=int, dest='sampleRate')
		self.parser.add_argument('-l', '--length', help='length of the measurement\
		 in seconds, %.1f s by default' % DEFAULT_LENGTH, default=DEFAULT_LENGTH,
		  type=float, dest='length')
		  
		self.args = self.parser.parse_args() 
		
	
    	def __init__(self):
    		
		self.parseArgs()
		
		self.setConnection()
		self.comm.startContSampling()
		self.X = []
		self.Y = []
		self.count = int(self.args.length * self.args.sampleRate)
		
		self.readData()
		self.writeToFile()
		self.drawPlot()
		
		self.stop()
		
		


GnuplotEdaqMeas()





